from src.models.ingredient_model import ingredient_model
from src.models.abstract_reference import abstract_reference
from src.validation.validator import validator

class recipe_model (abstract_reference):
    # Ингредиенты рецепта
    __ingredients = list()

    # Шаги приготовления рецепта
    __steps = list()


    # Валидатор
    __vtor = validator()


    def __init__(self, name):
        super().__init__(name)

    
    @staticmethod
    def create(name, ingredients:list=None, steps:list=None):
        """
            Фабричный метод для создания рецепта
        Args:
            ingredients?: список ингредиентов
            steps?: список шагов по приготовлению
        Returns:
            recipe_model: Созданный рецепт
        """

        recipe = recipe_model(name)
        
        recipe.ingredients = ingredients if ingredients is not None else list()
        recipe.steps = steps if steps is not None else list()

        return recipe


    @property
    def ingredients(self):
        """Ингредиенты рецепта"""

        return self.__ingredients
    
    @ingredients.setter
    def ingredients(self, ingredients: list):
        """
            Ингредиенты рецепта
        Args:
            ingredients (list): ингредиенты рецепта
        Raises:
            argument_exception: Несоответствие типа аргумента
            argument_exception: Один из членов коллекции не прошел валидацию
        """

        self.__vtor.check_type(ingredients, list)
        self.__vtor.check_collection_all(ingredients,
                                               lambda item: self.__vtor.check_type(item, ingredient_model))
        
        self.__ingredients = ingredients


    @property
    def steps(self):
        """Шаги рецепта"""

        return self.__steps
    
    @steps.setter
    def steps(self, steps: list):
        """
            Шаги рецепта
        Args:
            steps (list): шаги рецепта
        Raises:
            argument_exception: Несоответствие типа аргумента
            argument_exception: Один из членов коллекции не прошел валидацию
        """

        self.__vtor.check_type(steps, list)
        self.__vtor.check_collection_all(steps,
                                               lambda item: self.__vtor.check_type(item, str))
        self.__vtor.check_collection_all(steps,
                                               lambda item: self.__vtor.check_length_greater(item, 0))
        
        self.__steps = steps